#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>

#define MAXLINE 5120

int Socket(int family, int type, int protocol);
int Inet_aton(const char *strptr, struct in_addr *addrptr);
int Bind(int sockfd, const struct sockaddr *addr, socklen_t addrlen);
int Connect(int sockfd, const struct sockaddr *servaddr, socklen_t addrlen);
int Listen(int sockfd, int backlog);
int Accept(int sockfd, struct sockaddr *addr, socklen_t *addrlen);
ssize_t Send(int sockfd, const void *buf, size_t len, int flags);
ssize_t Recv(int sockfd, void *buf, size_t len, int flags);
int Close(int fd);

int main(int argc, char *argv[])
{
	int server_sock_listen, server_sock_data;
	struct sockaddr_in server_addr;
	char recv_msg[MAXLINE];

	if (argc != 2) {
		printf("Usage: %s port\n", argv[0]);
		exit(1);
	}

	/* 创建socket */
	server_sock_listen = Socket(AF_INET, SOCK_STREAM, 0);

	/* 指定服务器地址 */
	server_addr.sin_family = AF_INET;
	server_addr.sin_port = htons(atoi(argv[1]));
	server_addr.sin_addr.s_addr = htonl(INADDR_ANY); //INADDR_ANY表示本机所有IP地址
	memset(&server_addr.sin_zero, 0, sizeof(server_addr.sin_zero)); //零填充

	/* 绑定socket与地址 */
	Bind(server_sock_listen, (struct sockaddr *)&server_addr, sizeof(server_addr));

	/* 监听socket */
	Listen(server_sock_listen, 0);

	server_sock_data = Accept(server_sock_listen, NULL, NULL);

	while (1) {
		/* 接收并显示消息 */
		memset(recv_msg, 0, sizeof(recv_msg)); //接收数组置零
		if (Recv(server_sock_data, recv_msg, sizeof(recv_msg), 0) == 0) {	// 如果客户端关闭了连接就退出
			printf("Bye!\n");
			break;
		}
		printf("Recv: %s", recv_msg);
		if (strcasecmp(recv_msg, "bye\n") == 0) {							// 如果客户端输入的是 bye 那么就退出
			printf("Bye!\n");
			break;
		}

		/* 发送消息 */
		printf("Send: %s", recv_msg);
		Send(server_sock_data, recv_msg, strlen(recv_msg), 0);
	}

	/* 关闭数据socket */
	Close(server_sock_data);

	/* 关闭监听socket */
	Close(server_sock_listen);

	return 0;
}

int Socket(int family, int type, int protocol) {
	int n;
	if ((n = socket(family, type, protocol)) < 0) {
		perror("socket error");
		exit(1);
	}
	return n;
}
int Inet_aton(const char *strptr, struct in_addr *addrptr) {
	int n;
	if ((n = inet_aton(strptr, addrptr)) == 0) {
		perror("inet_aton error");
		exit(1);
	}
	return n;
}
int Bind(int sockfd, const struct sockaddr *addr, socklen_t addrlen) {
	int n;
	if ((n = bind(sockfd, addr, addrlen)) < 0) {
		perror("bind error");
		exit(1);
	}
	return n;
}
int Connect(int sockfd, const struct sockaddr *servaddr, socklen_t addrlen) {
	int n;
	if ((n = connect(sockfd, servaddr, addrlen)) < 0) {
		perror("connect error");
		exit(1);
	}
	return n;
}
int Listen(int sockfd, int backlog) {
	int n;
	if ((n = listen(sockfd, backlog)) < 0) {
		perror("listen error");
		exit(1);
	}
	return n;
}
int Accept(int sockfd, struct sockaddr *addr, socklen_t *addrlen) {
	int n;
	if ((n = accept(sockfd, addr, addrlen)) < 0) {
		perror("accept error");
		exit(1);
	}
	return n;
}
ssize_t Send(int sockfd, const void *buf, size_t len, int flags) {
	ssize_t n;
	if ((n = send(sockfd, buf, len, flags)) < 0) {
		perror("send error");
		exit(1);
	}
	return n;
}
ssize_t Recv(int sockfd, void *buf, size_t len, int flags) {
	ssize_t n;
	if ((n = recv(sockfd, buf, len, flags)) < 0) {
		perror("recv error");
		exit(1);
	}
	return n;
}
int Close(int fd) {
	int n;
	if ((n = close(fd)) < 0) {
		perror("close error");
		exit(1);
	}
	return n;
}